@@ -25,6 +25,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
@@ -82,7 +83,7 @@ static void on_app_btn_clicked( GtkButton* btn, MenuCacheApp* app )
 {
     lxlauncher_execute_app( gdk_screen_get_default(),
                                         NULL, menu_cache_app_get_exec(app), 
-                                        menu_cache_item_get_name(app), NULL, 
+                                        menu_cache_item_get_name(MENU_CACHE_ITEM(app)), NULL, 
                                         menu_cache_app_get_use_terminal(app),
                                         NULL );
 }
@@ -92,18 +93,16 @@ static void on_menu_item_properties(GtkMenuItem* item, MenuCacheApp* app)
     /* FIXME: if the source desktop is in AppDir other then default
      * applications dirs, where should we store the user-specific file?
     */
-    char* ifile = menu_cache_item_get_file_path(app);
+    char* ifile = menu_cache_item_get_file_path(MENU_CACHE_ITEM(app));
     char* ofile = g_build_filename(g_get_user_data_dir(), "applications",
-                                  menu_cache_item_get_file_basename(app), NULL);
-    char** argv[] = {
+                                  menu_cache_item_get_file_basename(MENU_CACHE_ITEM(app)), NULL);
+    char* argv[] = {
         "lxshortcut",
         "-i",
-        NULL,
+        ifile,
         "-o",
-        NULL,
+        ofile,
         NULL};
-    argv[2] = ifile;
-    argv[4] = ofile;
     g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL );
     g_free( ifile );
     g_free( ofile );
@@ -115,7 +114,7 @@ static gboolean on_app_btn_press_event(GtkWidget* btn, GdkEventButton* evt, Menu
     {
         char* tmp;
         GtkWidget* item;
-        GtkMenu* p = gtk_menu_new();
+        GtkWidget* p = gtk_menu_new();
         tmp = g_find_program_in_path("lxshortcut");
         if( tmp )
         {
@@ -125,20 +124,20 @@ static gboolean on_app_btn_press_event(GtkWidget* btn, GdkEventButton* evt, Menu
             */
             item = gtk_menu_item_new_with_mnemonic(_("_Customize"));
             g_signal_connect(item, "activate", G_CALLBACK(on_menu_item_properties), app);
-            gtk_menu_shell_append(p, item);
+            gtk_menu_shell_append(GTK_MENU_SHELL(p), item);
             g_free(tmp);
         }
         else
         {
             /* FIXME: since currently Properties is the only menu item,
              *        we don't popup the menu if it's empty */
-            gtk_widget_destroy(p);
+            gtk_widget_destroy(GTK_WIDGET(p));
             return FALSE;
         }
         g_signal_connect(p, "selection-done", G_CALLBACK(gtk_widget_destroy), NULL);
 
-        gtk_widget_show_all(p);
-        gtk_menu_popup(p, NULL, NULL, NULL, NULL, NULL, evt->time);
+        gtk_widget_show_all(GTK_WIDGET(p));
+        gtk_menu_popup(GTK_MENU(p), NULL, NULL, NULL, NULL, 0, evt->time);
         return TRUE;
     }
     return FALSE;
@@ -148,7 +147,7 @@ static void notebook_page_chdir( PageData* data, MenuCacheDir* dir );
 
 static void on_dir_btn_clicked( GtkButton* btn, PageData* data )
 {
-    MenuCacheDir* dir = (MenuCacheDir*)g_object_get_data( btn, "dir" );
+    MenuCacheDir* dir = (MenuCacheDir*)g_object_get_data( G_OBJECT(btn), "dir" );
     notebook_page_chdir( data, dir );
 }
 
@@ -166,31 +165,35 @@ static GtkWidget* add_btn( GtkWidget* table, const char* text, GdkPixbuf* icon,
     btn = gtk_button_new();
     gtk_widget_set_size_request( btn, button_size, -1 );
     if (!enable_key)
+#if GTK_CHECK_VERSION(2, 18, 0)
+        gtk_widget_set_can_focus(btn, FALSE );
+    gtk_widget_set_can_default(GTK_WIDGET(btn), FALSE );
+#else
         GTK_WIDGET_UNSET_FLAGS(btn, GTK_CAN_FOCUS );
     GTK_WIDGET_UNSET_FLAGS(btn, GTK_CAN_DEFAULT );
-
+#endif
     img = gtk_image_new_from_pixbuf( icon );
 
     box = gtk_vbox_new( FALSE, 2 );
-    gtk_box_pack_start( box, img, FALSE, TRUE, 2 );
+    gtk_box_pack_start( GTK_BOX(box), img, FALSE, TRUE, 2 );
 
     label = gtk_label_new( text );
     gtk_widget_show( label );
     gtk_widget_set_size_request( label, button_size - 10, -1 );
-    gtk_label_set_line_wrap_mode( label, PANGO_WRAP_WORD_CHAR );
-    gtk_label_set_line_wrap( label, TRUE );
-    gtk_label_set_justify( label, GTK_JUSTIFY_CENTER );
-    gtk_misc_set_alignment( label, 0.5, 0 );
-    gtk_misc_set_padding( label, 0, 0 );
-    gtk_box_pack_start( box, label, TRUE, TRUE, 0 );
+    gtk_label_set_line_wrap_mode( GTK_LABEL(label), PANGO_WRAP_WORD_CHAR );
+    gtk_label_set_line_wrap( GTK_LABEL(label), TRUE );
+    gtk_label_set_justify( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
+    gtk_misc_set_alignment( GTK_MISC(label), 0.5, 0 );
+    gtk_misc_set_padding( GTK_MISC(label), 0, 0 );
+    gtk_box_pack_start( GTK_BOX(box), label, TRUE, TRUE, 0 );
 
-    gtk_container_add( btn, box );
+    gtk_container_add( GTK_CONTAINER(btn), box );
 
-    gtk_button_set_relief( btn, GTK_RELIEF_NONE );
+    gtk_button_set_relief( GTK_BUTTON(btn), GTK_RELIEF_NONE );
     gtk_widget_set_size_request( btn, button_size, button_size );
     gtk_widget_show_all( btn );
 
-    gtk_container_add( table, btn );
+    gtk_container_add( GTK_CONTAINER(table), btn );
     gtk_widget_realize( btn );
 
     gtk_widget_set_tooltip_text(btn, tip);
@@ -203,12 +206,12 @@ static GtkWidget* add_btn( GtkWidget* table, const char* text, GdkPixbuf* icon,
 
     /* get border of GtkButtons */
     gtk_widget_style_get(btn, "focus-line-width", &fw, "focus-padding", &fp, NULL);
-    btn_border = 2 * (gtk_container_get_border_width(btn) + btn->style->xthickness + fw + fp);
+    btn_border = 2 * (gtk_container_get_border_width(GTK_CONTAINER(btn)) + gtk_widget_get_style(btn)->xthickness + fw + fp);
     gtk_widget_style_get(btn, "inner-border", &inner_border, NULL);
     if( inner_border )
         btn_border += (inner_border->left + inner_border->right);
     /* padding of vbox should be added. */
-    btn_border += 2 * gtk_container_get_border_width(box);
+    btn_border += 2 * gtk_container_get_border_width(GTK_CONTAINER(box));
 
     gtk_widget_size_request( label, &req );
 
@@ -216,12 +219,18 @@ static GtkWidget* add_btn( GtkWidget* table, const char* text, GdkPixbuf* icon,
     if( req.width > (button_size - btn_border) )
     {
         gtk_widget_set_size_request( label, button_size - btn_border, -1 );
-        gtk_label_set_line_wrap_mode( label, PANGO_WRAP_WORD_CHAR );
-        gtk_label_set_line_wrap( label, TRUE );
+        gtk_label_set_line_wrap_mode( GTK_LABEL(label), PANGO_WRAP_WORD_CHAR );
+        gtk_label_set_line_wrap( GTK_LABEL(label), TRUE );
     }
 
     gtk_widget_set_app_paintable( btn, TRUE );
+#if GTK_CHECK_VERSION(3, 0, 0)
+    GdkRGBA color;
+    gdk_rgba_parse(&color, "none");
+    gtk_widget_override_background_color ((GtkWidget*)btn, GTK_STATE_FLAG_NORMAL, &color);
+#else
     gdk_window_set_back_pixmap( ((GtkWidget*)btn)->window, NULL, TRUE );
+#endif
     return btn;
 }
 
@@ -229,17 +238,17 @@ static void add_dir_btn( PageData* data, MenuCacheDir* dir )
 {
     GdkPixbuf* icon;
     GtkWidget* btn;
-    const char* icon_name = menu_cache_item_get_icon( dir );
+    const char* icon_name = menu_cache_item_get_icon( MENU_CACHE_ITEM(dir) );
     if( !icon_name )
         icon_name = "folder";
 
     icon = lxlauncher_load_icon( icon_name, img_size, TRUE );
 
-    btn = add_btn( data->table, menu_cache_item_get_name(dir), icon, menu_cache_item_get_comment(dir) );
+    btn = add_btn( data->table, menu_cache_item_get_name(MENU_CACHE_ITEM(dir)), icon, menu_cache_item_get_comment(MENU_CACHE_ITEM(dir)) );
     if( icon )
         g_object_unref( icon );
 
-    g_object_set_data( btn, "dir", dir );
+    g_object_set_data( G_OBJECT(btn), "dir", dir );
     g_signal_connect( btn, "clicked", G_CALLBACK(on_dir_btn_clicked), data );
 }
 
@@ -247,14 +256,14 @@ static void add_app_btn( GtkWidget* table, MenuCacheApp* app )
 {
     GdkPixbuf* icon;
     GtkWidget* btn;
-    const char* icon_name = menu_cache_item_get_icon( app );
+    const char* icon_name = menu_cache_item_get_icon( MENU_CACHE_ITEM(app) );
 
     if( !icon_name )
         icon_name = "application-x-executable";
 
     icon = lxlauncher_load_icon( icon_name, img_size, TRUE );
 
-    btn = add_btn( table, menu_cache_item_get_name(app), icon, menu_cache_item_get_comment(app) );
+    btn = add_btn( table, menu_cache_item_get_name(MENU_CACHE_ITEM(app)), icon, menu_cache_item_get_comment(MENU_CACHE_ITEM(app)) );
 
     if( icon )
         g_object_unref( icon );
@@ -262,19 +271,43 @@ static void add_app_btn( GtkWidget* table, MenuCacheApp* app )
     g_signal_connect( btn, "button-press-event", G_CALLBACK(on_app_btn_press_event), app );
 }
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+static gboolean on_viewport_draw( GtkWidget* w, cairo_t *cr1, gpointer data )
+{
+    GtkWidget* scroll = gtk_widget_get_parent(w);
+	GtkStyleContext *style = gtk_widget_get_style_context(scroll);
+
+    cairo_t *cr;
+
+    cr = gdk_cairo_create (gtk_widget_get_window(w));
+    cairo_rectangle(cr, 0, 0, 0, 0 );
+    cairo_set_source_rgb(cr, 184.0/256, 215.0/256, 235.0/256);
+    cairo_fill(cr);
+
+	gtk_render_background(style,cr,0,0,
+			gtk_widget_get_allocated_width(scroll),
+			gtk_widget_get_allocated_height(scroll));
+
+
+    cairo_destroy(cr);
+    return FALSE;
+
+}
+#else
 static gboolean on_viewport_expose( GtkWidget* w, GdkEventExpose* evt, gpointer data )
 {
     GObjectClass* oc = G_OBJECT_GET_CLASS(w);
     GtkWidgetClass* wc = (GtkWidgetClass*)g_type_class_peek_parent( oc );
+/*
     GdkPixmap* pixmap = (GdkPixmap*)data;
-
+*/
     if( GTK_WIDGET_DRAWABLE(w) && evt->window == ((GtkViewport*)w)->bin_window )
     {
         cairo_t *cr;
         cairo_pattern_t* pat;
 
         GtkWidget* scroll = gtk_widget_get_parent(w);
-        GtkAdjustment* vadj = gtk_scrolled_window_get_vadjustment(scroll);
+        GtkAdjustment* vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll));
 
         cr = gdk_cairo_create (evt->window);
         //pat = cairo_pattern_create_linear( 0, gtk_adjustment_get_value(vadj), 0, w->allocation.height + gtk_adjustment_get_value(vadj) );
@@ -303,37 +336,41 @@ static gboolean on_viewport_expose( GtkWidget* w, GdkEventExpose* evt, gpointer
     }
 
     // call handler of tha parent GtkContainer class to propagate the event to children
+
     (* wc->expose_event) (w, evt);
     return TRUE;
 }
+#endif
 
 static gboolean on_scroll( GtkAdjustment* adj, PageData* data )
 {
     // Dirty hacks used to force pseudo-transparent background
     gtk_widget_queue_draw( data->table );
+    return TRUE;
 }
 
 // Dirty hacks used to reduce unnecessary redrew during scroll
 static gboolean on_scroll_change_val( GtkRange* scroll, GtkScrollType type, gdouble value, PageData* data )
 {
+
     GtkAdjustment* adj = gtk_range_get_adjustment(scroll);
 
     if( type == GTK_SCROLL_JUMP )
     {
-        if( ABS( adj->value - value ) < button_size / 2 )
+        if( ABS( gtk_adjustment_get_value(adj) - value ) < button_size / 2 )
         {
-            if( adj->value > value )    // upward
+            if( gtk_adjustment_get_value(adj) > value )    // upward
             {
-                if( (adj->value - adj->lower) < button_size / 2 )
+                if( (gtk_adjustment_get_value(adj) - gtk_adjustment_get_lower(adj)) < button_size / 2 )
                 {
-                    gtk_adjustment_set_value( adj, adj->lower );
+                    gtk_adjustment_set_value( adj, gtk_adjustment_get_lower(adj) );
                 }
             }
             else // downward
             {
-                if( (adj->upper - adj->value) < button_size / 2 )
+                if( (gtk_adjustment_get_upper(adj) - gtk_adjustment_get_value(adj)) < button_size / 2 )
                 {
-                    gtk_adjustment_set_value( adj, adj->lower );
+                    gtk_adjustment_set_value( adj, gtk_adjustment_get_lower(adj) );
                 }
             }
             return TRUE;
@@ -344,13 +381,13 @@ static gboolean on_scroll_change_val( GtkRange* scroll, GtkScrollType type, gdou
 
 static char* menu_dir_to_path( MenuCacheDir* dir )
 {
-    if( menu_cache_item_get_parent(dir) == root_dir )
-        return g_strdup( menu_cache_item_get_name(dir) );
+    if( menu_cache_item_get_parent(MENU_CACHE_ITEM(dir)) == root_dir )
+        return g_strdup( menu_cache_item_get_name(MENU_CACHE_ITEM(dir)) );
     else
     {
-        char* parent = menu_dir_to_path( menu_cache_item_get_parent(dir) );
+        char* parent = menu_dir_to_path( menu_cache_item_get_parent(MENU_CACHE_ITEM(dir)) );
         char* ret;
-        ret = g_strconcat( parent, " > ", menu_cache_item_get_name(dir), NULL );
+        ret = g_strconcat( parent, " > ", menu_cache_item_get_name(MENU_CACHE_ITEM(dir)), NULL );
         g_free( parent );
         return ret;
     }
@@ -367,19 +404,19 @@ static PageData* notebook_page_from_dir( MenuCacheDir* dir )
     PageData* page_data;
 
     // get toplevel parent dir
-    while( menu_cache_item_get_parent(top) != root_dir )
-        top = menu_cache_item_get_parent(top);
+    while( menu_cache_item_get_parent(MENU_CACHE_ITEM(top)) != root_dir )
+        top = menu_cache_item_get_parent(MENU_CACHE_ITEM(top));
 
-    n = gtk_notebook_get_n_pages( notebook );
+    n = gtk_notebook_get_n_pages( GTK_NOTEBOOK(notebook) );
     for( i = 0; i < n; ++i )
     {
 
         MenuCacheDir* top2;
-        page = gtk_notebook_get_nth_page( notebook, i );
-        page_data = (PageData*)g_object_get_data(page, "page");
+        page = gtk_notebook_get_nth_page( GTK_NOTEBOOK(notebook), i );
+        page_data = (PageData*)g_object_get_data(G_OBJECT(page), "page");
         top2 = page_data->dir;
-        while( menu_cache_item_get_parent(top2) != root_dir )
-            top2 = menu_cache_item_get_parent(top2);
+        while( menu_cache_item_get_parent(MENU_CACHE_ITEM(top2)) != root_dir )
+            top2 = menu_cache_item_get_parent(MENU_CACHE_ITEM(top2));
 
         if( top == top2 )
             return page_data;
@@ -393,16 +430,16 @@ static gboolean reload_apps()
     PageData* page_data;
     GtkWidget* page;
 
-    int n = gtk_notebook_get_n_pages( notebook );
+    int n = gtk_notebook_get_n_pages( GTK_NOTEBOOK(notebook) );
     char** page_paths = g_new0(char*, n + 1);
-    int cur_page = gtk_notebook_get_current_page(notebook);
+    int cur_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
 
     for( i = 0; i < n; ++i )
     {
-        page = gtk_notebook_get_nth_page( notebook, 0 );
-        page_data = (PageData*)g_object_get_data( page, "page" );
+        page = gtk_notebook_get_nth_page( GTK_NOTEBOOK(notebook), 0 );
+        page_data = (PageData*)g_object_get_data( G_OBJECT(page), "page" );
         page_paths[i] = g_strdup(page_data->dir_path);
-        gtk_notebook_remove_page( notebook, 0 );
+        gtk_notebook_remove_page( GTK_NOTEBOOK(notebook), 0 );
     }
 
     // rebuild every pages
@@ -417,8 +454,8 @@ static gboolean reload_apps()
             MenuCacheDir* top = dir;
             // get toplevel parent dir
 
-            while( menu_cache_item_get_parent(top) != root_dir )
-                top = menu_cache_item_get_parent(top);
+            while( menu_cache_item_get_parent(MENU_CACHE_ITEM(top)) != root_dir )
+                top = menu_cache_item_get_parent(MENU_CACHE_ITEM(top));
 
             // find notebook page containing the top dir
             page_data = notebook_page_from_dir( top );
@@ -430,7 +467,7 @@ static gboolean reload_apps()
                 notebook_page_chdir( page_data, dir );
 
             if( i == cur_page )
-                gtk_notebook_set_current_page( notebook, cur_page );
+                gtk_notebook_set_current_page( GTK_NOTEBOOK(notebook), cur_page );
         }
     }
     g_strfreev( page_paths );
@@ -442,7 +479,7 @@ void on_menu_tree_changed( MenuCache *tree, gpointer  user_data )
     // some changes happened in applications dirs
     // reload is needed
     // g_debug( "file changed" );
-    menu_cache_item_unref(root_dir);
+    menu_cache_item_unref(MENU_CACHE_ITEM(root_dir));
     root_dir = menu_cache_ref(menu_cache_get_root_dir(tree));
     reload_apps();
 }
@@ -454,9 +491,9 @@ GdkFilterReturn evt_filter(GdkXEvent *xevt, GdkEvent *evt, gpointer data)
     if(xev->type == PropertyNotify && xev->xproperty.atom == atom_NET_WORKAREA )
     {
         GdkRectangle working_area;
-        get_working_area( gdk_drawable_get_screen(evt->any.window), &working_area );
-        gtk_window_move( main_window, working_area.x, working_area.y );
-        gtk_window_resize( main_window, working_area.width, working_area.height );
+        get_working_area( gdk_window_get_screen(evt->any.window), &working_area );
+        gtk_window_move( GTK_WINDOW(main_window), working_area.x, working_area.y );
+        gtk_window_resize( GTK_WINDOW(main_window), working_area.width, working_area.height );
     }
     return GDK_FILTER_CONTINUE;
 }
@@ -494,15 +531,15 @@ static void notebook_page_chdir( PageData* data, MenuCacheDir* dir )
     MenuCacheDir* parent_dir;
 
     if(data->dir)
-        menu_cache_item_unref(data->dir);
-    data->dir = menu_cache_item_ref(dir);
+        menu_cache_item_unref(MENU_CACHE_ITEM(data->dir));
+    data->dir = menu_cache_item_ref(MENU_CACHE_ITEM(dir));
     g_free(data->dir_path);
     data->dir_path = menu_cache_dir_make_path( dir );
 
     // destroy old buttons
-    gtk_container_foreach( data->table, gtk_widget_destroy, NULL );
+    gtk_container_foreach( GTK_CONTAINER(data->table), gtk_widget_destroy, NULL );
 
-    gtk_container_forall( data->go_up_bar, gtk_widget_destroy, NULL );
+    gtk_container_forall( GTK_CONTAINER(data->go_up_bar), gtk_widget_destroy, NULL );
 
     if( G_UNLIKELY( !dir ) )
         return;
@@ -516,7 +553,7 @@ static void notebook_page_chdir( PageData* data, MenuCacheDir* dir )
             add_dir_btn( data, (MenuCacheDir*)item );
         else if( type == MENU_CACHE_TYPE_APP )
         {
-            if( ! menu_cache_app_get_is_visible(item, SHOW_IN_LXDE) )
+            if( ! menu_cache_app_get_is_visible(MENU_CACHE_APP(item), SHOW_IN_LXDE) )
                 continue;
             add_app_btn( data->table, (MenuCacheApp*)item );
         }
@@ -527,7 +564,7 @@ static void notebook_page_chdir( PageData* data, MenuCacheDir* dir )
     if( parent_dir != root_dir )   // if dir has parent, not top-level group
     {
         GtkWidget* label;
-        char* text = g_strdup_printf( _("Go back to \"%s\""), menu_cache_item_get_name(parent_dir) );
+        char* text = g_strdup_printf( _("Go back to \"%s\""), menu_cache_item_get_name(MENU_CACHE_ITEM(parent_dir)) );
         GtkWidget* btn = gtk_button_new_with_label( text );
         g_free( text );
 
@@ -536,18 +573,18 @@ static void notebook_page_chdir( PageData* data, MenuCacheDir* dir )
         label = gtk_label_new( dir_path );
         g_free( dir_path );
 
-        gtk_misc_set_alignment( label, 0.0, 0.5 );
-        g_object_set_data( btn, "dir", parent_dir );
+        gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5 );
+        g_object_set_data( G_OBJECT(btn), "dir", parent_dir );
         g_signal_connect( btn, "clicked", on_dir_btn_clicked, data );
-        gtk_button_set_image( btn, gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_BUTTON ) );
+        gtk_button_set_image( GTK_BUTTON(btn), gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_BUTTON ) );
 
         gtk_box_pack_start( data->go_up_bar, label, TRUE, TRUE, 4 );
         gtk_box_pack_start( data->go_up_bar, btn, FALSE, TRUE, 2 );
-        gtk_widget_show_all( data->go_up_bar );
+        gtk_widget_show_all( GTK_WIDGET(data->go_up_bar) );
     }
     else
     {
-        gtk_widget_hide( data->go_up_bar );
+        gtk_widget_hide( GTK_WIDGET(data->go_up_bar) );
     }
 }
 
@@ -556,7 +593,7 @@ static void page_data_free( PageData* data )
     if( data->background )
         g_object_unref( data->background );
     if(data->dir)
-        menu_cache_item_unref( data->dir );
+        menu_cache_item_unref( MENU_CACHE_ITEM(data->dir) );
     g_free(data->dir_path);
     g_free( data );
 }
@@ -577,8 +614,6 @@ static void create_notebook_pages()
         GtkWidget* image;
         GtkWidget* go_up_bar = gtk_hbox_new( FALSE, 2 );
         GdkPixbuf* pixbuf=NULL;
-        GdkPixmap* pixmap;
-        GdkGC *pixmap_gc=NULL;
         char* file;
         PageData* page_data;
         MenuCacheDir* dir = (MenuCacheDir*)l->data;
@@ -587,50 +622,60 @@ static void create_notebook_pages()
             continue;
 
         page_data = g_new0( PageData, 1 );
-        g_object_set_data_full( page_vbox, "page", page_data, page_data_free );
+        g_object_set_data_full( G_OBJECT(page_vbox), "page", page_data, page_data_free );
 
         label = gtk_hbox_new( FALSE, 2 );
 
-        gtk_scrolled_window_set_policy(scroll, GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC );
+        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC );
 
         // Very bad dirty hacks used to force gtk+ to draw transparent background
-        GtkRange* range = gtk_scrolled_window_get_vscrollbar(scroll);
+        GtkRange* range = gtk_scrolled_window_get_vscrollbar(GTK_SCROLLED_WINDOW(scroll));
         //gtk_range_set_update_policy( range, GTK_UPDATE_DELAYED );
         g_signal_connect( range, "change-value", G_CALLBACK(on_scroll_change_val), page_data );
-        adj = gtk_scrolled_window_get_vadjustment(scroll);
+        adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll));
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_adjustment_set_step_increment(adj, button_size / 3);
+        gtk_adjustment_set_page_increment(adj, button_size / 2);
+#else
         adj->step_increment = button_size / 3;
         adj->page_increment = button_size / 2;
+#endif
         gtk_adjustment_changed( adj );
         g_signal_connect( adj, "value-changed", G_CALLBACK(on_scroll), page_data );
 
         // create label
-        image = gtk_image_new_from_icon_name( menu_cache_item_get_icon(dir), GTK_ICON_SIZE_MENU );
+        image = gtk_image_new_from_icon_name( menu_cache_item_get_icon(MENU_CACHE_ITEM(dir)), GTK_ICON_SIZE_MENU );
 
-        gtk_box_pack_start( label, image, FALSE, TRUE, 2 );
-        gtk_box_pack_start( label, gtk_label_new( menu_cache_item_get_name(dir) ), FALSE, TRUE, 2 );
+        gtk_box_pack_start( GTK_BOX(label), image, FALSE, TRUE, 2 );
+        gtk_box_pack_start( GTK_BOX(label), gtk_label_new( menu_cache_item_get_name(MENU_CACHE_ITEM(dir)) ), FALSE, TRUE, 2 );
         gtk_widget_show_all(label);
 
         // gtk_container_set_border_width( page_vbox, 4 );
-        exo_wrap_table_set_col_spacing( table, 8 );
+        exo_wrap_table_set_col_spacing( EXO_WRAP_TABLE(table), 8 );
 
         viewport = gtk_viewport_new( NULL, NULL );
-        gtk_container_add( viewport, table );
-        gtk_container_add( scroll, viewport );
+        gtk_container_add( GTK_CONTAINER(viewport), table );
+        gtk_container_add( GTK_CONTAINER(scroll), GTK_WIDGET(viewport) );
         gtk_widget_show_all( scroll );
 
-        gtk_box_pack_start( page_vbox, go_up_bar, FALSE, TRUE, 0 );
-        gtk_box_pack_start( page_vbox, scroll, TRUE, TRUE, 0 );
+        gtk_box_pack_start( GTK_BOX(page_vbox), go_up_bar, FALSE, TRUE, 0 );
+        gtk_box_pack_start( GTK_BOX(page_vbox), scroll, TRUE, TRUE, 0 );
         gtk_widget_show_all( page_vbox );
 
-        gtk_notebook_append_page( notebook, page_vbox, label );
+        gtk_notebook_append_page( GTK_NOTEBOOK(notebook), page_vbox, label );
 
         // set background
-        gtk_widget_set_app_paintable( viewport, TRUE );
+        gtk_widget_set_app_paintable( GTK_WIDGET(viewport), TRUE );
+#if GTK_CHECK_VERSION(3, 0, 0)
+        g_signal_connect( viewport, "draw", G_CALLBACK(on_viewport_draw), NULL );
+#else
 /*
         file = g_build_filename( BACKGROUND_DIR, groups[i].background, NULL );
         pixbuf = gdk_pixbuf_new_from_file( file, NULL );
         g_free( file );
 */
+        GdkPixmap* pixmap;
+        GdkGC *pixmap_gc=NULL;
         if( pixbuf )
         {
             pixmap = gdk_pixmap_new( gdk_get_default_root_window(), gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf), -1 );
@@ -642,10 +687,11 @@ static void create_notebook_pages()
                                     GDK_RGB_DITHER_NORMAL, 0, 0 );
             g_object_unref( pixbuf );
 
-            g_object_weak_ref( viewport, (GWeakNotify)g_object_unref, pixmap );
+            g_object_weak_ref( G_OBJECT(viewport), (GWeakNotify)g_object_unref, pixmap );
             g_object_unref(pixmap_gc);
         }
         g_signal_connect( viewport, "expose_event", G_CALLBACK(on_viewport_expose), pixmap );
+#endif
 
         page_data->page_vbox = page_vbox;
         page_data->go_up_bar = go_up_bar;
@@ -662,18 +708,38 @@ gchar* get_xdg_config_file(const char *name) {
     gchar *file;
 
     file = g_build_filename(user_dir, name, NULL);
-    if (g_file_test(file, G_FILE_TEST_EXISTS) == TRUE) {
-	return file;
-    }
+    if (g_file_test(file, G_FILE_TEST_EXISTS) == TRUE)
+    	return file;
     free(file);
 
     for (dir = system_dirs; *dir; ++dir ) {
-	file = g_build_filename(*dir, name, NULL);
-	if (g_file_test(file, G_FILE_TEST_EXISTS) == TRUE) {
-	    return file;
-	}
-	free(file);
+        file = g_build_filename(*dir, name, NULL);
+        if (g_file_test(file, G_FILE_TEST_EXISTS) == TRUE)
+            return file;
+	    free(file);
     }
+    return NULL;
+}
+
+static void on_menu_cache_reload(MenuCache* mc, gpointer user_data)
+{
+    GMainLoop* mainloop = (GMainLoop*)user_data;
+    g_main_loop_quit(mainloop);
+}
+
+MenuCache* _menu_cache_lookup_sync( const char* menu_name )
+{
+    MenuCache* mc = menu_cache_lookup(menu_name);
+    /* ensure that the menu cache is loaded */
+    if(! menu_cache_get_root_dir(mc)) /* if it's not yet loaded */
+    {
+        GMainLoop* mainloop = g_main_loop_new(NULL, FALSE);
+        gpointer notify_id = menu_cache_add_reload_notify(mc, on_menu_cache_reload, mainloop);
+        g_main_loop_run(mainloop);
+        g_main_loop_unref(mainloop);
+        menu_cache_remove_reload_notify(mc, notify_id);
+    }
+    return mc;
 }
 
 int main(int argc, char** argv)
@@ -683,6 +749,7 @@ int main(int argc, char** argv)
     GList* l;
     gboolean enable_key=0;
     GtkSettings *settings;
+    const gchar* prefix = g_getenv("XDG_MENU_PREFIX");
 
 #ifdef ENABLE_NLS
     bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
@@ -695,13 +762,14 @@ int main(int argc, char** argv)
     GError *error = NULL;
     gchar *config_file = get_xdg_config_file(CONFIG_FILE);
     if (config_file &&
-	g_key_file_load_from_file(key_file,
-				  config_file,
-				  G_KEY_FILE_NONE,
-				  &error)) {
-	printf("Loaded %s\n", config_file);
-    } else {
-	perror("Error loading " CONFIG_FILE);
+        g_key_file_load_from_file(key_file,
+                      config_file,
+                      G_KEY_FILE_NONE,
+                      &error)) {
+    	printf("Loaded %s\n", config_file);
+    }
+    else {
+    	perror("Error loading " CONFIG_FILE);
     }
     if (config_file)
 	free(config_file);
@@ -709,42 +777,59 @@ int main(int argc, char** argv)
     gtk_init( &argc, &argv );
 
     // Add application specific properties
-    gtk_settings_install_property(g_param_spec_boolean("lxlauncher-enable-key",
+    gtk_settings_install_property(
+                             g_param_spec_boolean("lxlauncher-enable-key",
                             _("Enable key navigation"),
-                            _("Allow users to use up/down/left/right/tab/enter keys to operate the lxlaucher"),
-                            FALSE,GTK_ARG_READWRITE));
+                            _("Allow users to use up/down/left/right/tab/enter keys to operate the lxlauncher"),
+                            FALSE,
+                            G_PARAM_READWRITE));
     // set up themes for notebook
     gchar* gtkrc_file = get_xdg_config_file("lxlauncher/gtkrc");
-    gtk_rc_parse(gtkrc_file);
     if (gtkrc_file) {
-	free(gtkrc_file);
+        gtk_rc_parse(gtkrc_file);
+    	free(gtkrc_file);
     }
 
     button_size = g_key_file_get_integer(key_file, "Main", "BUTTON_SIZE", NULL);
     img_size = g_key_file_get_integer(key_file, "Main", "IMG_SIZE", NULL);
 
     // to prevent from going without configure file
-    if(!button_size) button_size = BUTTON_SIZE_FALLBACK;
-    if(!img_size) img_size = IMG_SIZE_FALLBACK;
+    if(!button_size)
+        button_size = BUTTON_SIZE_FALLBACK;
+    if(!img_size)
+        img_size = IMG_SIZE_FALLBACK;
 
     icon_size = gtk_icon_size_register( "ALIcon", img_size, img_size );
 
     main_window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
-    gtk_window_move( main_window, 0, 0 );
-    gtk_window_set_skip_pager_hint( main_window, TRUE );
-    gtk_window_set_skip_taskbar_hint( main_window, TRUE );
+    gtk_window_move( GTK_WINDOW(main_window), 0, 0 );
+    gtk_window_set_skip_pager_hint( GTK_WINDOW(main_window), TRUE );
+    gtk_window_set_skip_taskbar_hint( GTK_WINDOW(main_window), TRUE );
 
     gtk_widget_realize( main_window );
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gdk_window_set_keep_below( gtk_widget_get_window(main_window), TRUE );
+    //gdk_window_set_decorations( main_window->window );
+    gdk_window_set_type_hint( gtk_widget_get_window(main_window), GDK_WINDOW_TYPE_HINT_DESKTOP );
+#else
     gdk_window_set_keep_below( main_window->window, TRUE );
     //gdk_window_set_decorations( main_window->window );
     gdk_window_set_type_hint( main_window->window, GDK_WINDOW_TYPE_HINT_DESKTOP );
-    gtk_window_set_position( main_window, GTK_WIN_POS_NONE );
+#endif
+    gtk_window_set_position( GTK_WINDOW(main_window), GTK_WIN_POS_NONE );
     //gtk_window_set_gravity(GDK_GRAVITY_STATIC );
 
+    /* Remove resize grip with GTK3
+    */
+#if GTK_CHECK_VERSION(3, 0, 0)
+    gtk_window_set_has_resize_grip(GTK_WINDOW(main_window), FALSE);
+#else
+#endif
+
     g_signal_connect(main_window, "delete-event", G_CALLBACK(window_delete), NULL);
 
-    atom_NET_WORKAREA = XInternAtom( GDK_DISPLAY(), "_NET_WORKAREA", True);;
-    XSelectInput(GDK_DISPLAY(), GDK_WINDOW_XID(gtk_widget_get_root_window(main_window)), PropertyChangeMask );
+    atom_NET_WORKAREA = XInternAtom( GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_NET_WORKAREA", True);
+    XSelectInput(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), GDK_WINDOW_XID(gtk_widget_get_root_window(main_window)), PropertyChangeMask );
     gdk_window_add_filter( gtk_widget_get_root_window(main_window), evt_filter, NULL );
 
     notebook = gtk_notebook_new();
@@ -752,26 +837,37 @@ int main(int argc, char** argv)
     g_object_get(settings, "lxlauncher-enable-key", &enable_key,NULL);
     
     if (!enable_key)
+#if GTK_CHECK_VERSION(2, 18, 0)
+        gtk_widget_set_can_focus(notebook, FALSE );
+#else
         GTK_WIDGET_UNSET_FLAGS(notebook, GTK_CAN_FOCUS );
+#endif
     gtk_container_add( (GtkContainer*)main_window, notebook );
+    
+    if ((prefix == NULL) || (strcmp(prefix, "lxde-") == 0))
+        g_setenv("XDG_MENU_PREFIX", "lxlauncher-", TRUE);
 
-    g_setenv("XDG_MENU_PREFIX", "lxlauncher-", TRUE);
-    menu_tree = menu_cache_lookup( "applications.menu" );
+    menu_tree = _menu_cache_lookup_sync( "applications.menu" );
+    if(!menu_tree)
+    {
+        g_print("Unable to load application menu\n");
+        return 1;
+    }
+    root_dir = menu_cache_item_ref(MENU_CACHE_ITEM(menu_cache_get_root_dir( menu_tree )));
     reload_notify_id = menu_cache_add_reload_notify( menu_tree, on_menu_tree_changed, NULL );
-    root_dir = menu_cache_ref(menu_cache_get_root_dir( menu_tree ));
 
     create_notebook_pages();
 
     get_working_area( gtk_widget_get_screen(main_window), &working_area );
-    gtk_window_move( main_window, working_area.x, working_area.y );
-    gtk_window_resize( main_window, working_area.width, working_area.height );
+    gtk_window_move( GTK_WINDOW(main_window), working_area.x, working_area.y );
+    gtk_window_resize( GTK_WINDOW(main_window), working_area.width, working_area.height );
 
     gtk_widget_show_all( main_window );
     gtk_main();
 
     gdk_window_remove_filter( gtk_widget_get_root_window(main_window), evt_filter, NULL );
     menu_cache_remove_reload_notify( menu_tree, reload_notify_id );
-    menu_cache_item_unref( root_dir );
+    menu_cache_item_unref( MENU_CACHE_ITEM(root_dir) );
     menu_cache_unref( menu_tree );
 
     return 0;
